from threading import Thread, Lock
import time


class BulkQueue():

    def __init__(self, es_client, threshold=10, flush_time=30):
        self.es_client = es_client
        self.threshold = threshold
        self.list = list()
        self.flush_time = flush_time
        self.flush_thread = Thread(target=self.flush_periodically)
        self.flush_thread.daemon = True
        self.lock = Lock()
        self.running = False

    def start_thread(self):
        self.stopping_thread = False
        self.running = True
        self.flush_thread.start()

    def stop_thread(self):
        self.stopping_thread = True
        self.running = False

    def flush_periodically(self):
        while True and not self.stopping_thread:
            time.sleep(self.flush_time)
            self.flush()

    def started(self):
        return self.running

    def add(self, index, body, doc_id=None):
        """ Add event to queue, flushing if we hit the threshold """
        bulk_doc = {
            "_index": index,
            "_id": doc_id,
            "_source": body
        }
        self.lock.acquire()
        try:
            self.list.append(bulk_doc)
        finally:
            self.lock.release()
        if self.size() >= self.threshold:
            self.flush()

    def size(self):
        """ Size of the queue structure """
        return len(self.list)

    def flush(self):
        """ Write all stored events to ES """
        self.es_client.save_documents(self.list)
        self.lock.acquire()
        try:
            self.list = list()
        finally:
            self.lock.release()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# Copyright (c) 2017 Mozilla Corporation


from elasticsearch_dsl import Q


def WildcardMatch(key, value):
    return Q('wildcard', **{key: value})



from .aggregated_results import AggregatedResults
from .aggregation import Aggregation
from .boolean_match import BooleanMatch
from .exists_match import ExistsMatch
from .phrase_match import PhraseMatch
from .query_string_match import QueryStringMatch
from .range_match import RangeMatch
from .search_query import SearchQuery
from .simple_results import SimpleResults
from .term_match import TermMatch
from .terms_match import TermsMatch
from .wildcard_match import WildcardMatch
from .less_than_match import LessThanMatch



def toUnicode(obj, encoding='utf-8'):
    if type(obj) in [int, long, float, complex]:
        # likely a number, convert it to string to get to unicode
        obj = str(obj)
    if isinstance(obj, basestring):
        if not isinstance(obj, unicode):
            obj = unicode(obj, encoding)
    return obj



from datetime import datetime
from dateutil.parser import parse
import pytz
import math
import tzlocal

LOCAL_TIMEZONE = tzlocal.get_localzone()


def toUTC(suspectedDate):
    '''make a UTC date out of almost anything'''
    utc = pytz.UTC
    objDate = None
    if type(suspectedDate) == datetime:
        objDate = suspectedDate
    elif type(suspectedDate) == float:
        if suspectedDate <= 0:
            objDate = datetime(1970, 1, 1)
        else:
            # This breaks in the year 2286
            EPOCH_MAGNITUDE = 9
            magnitude = int(math.log10(int(suspectedDate)))
            if magnitude > EPOCH_MAGNITUDE:
                suspectedDate = suspectedDate / 10 ** (magnitude - EPOCH_MAGNITUDE)
            objDate = datetime.fromtimestamp(suspectedDate, LOCAL_TIMEZONE)
    elif str(suspectedDate).isdigit():
        if int(str(suspectedDate)) <= 0:
            objDate = datetime(1970, 1, 1)
        else:
            # epoch? but seconds/milliseconds/nanoseconds (lookin at you heka)
            epochDivisor = int(str(1) + '0' * (len(str(suspectedDate)) % 10))
            objDate = datetime.fromtimestamp(float(suspectedDate / epochDivisor), LOCAL_TIMEZONE)
    elif type(suspectedDate) in (str, unicode):
        # try to parse float or negative number from string:
        objDate = None
        try:
            suspected_float = float(suspectedDate)
            if suspected_float <= 0:
                objDate = datetime(1970, 1, 1)
        except ValueError:
            pass
        if objDate is None:
            objDate = parse(suspectedDate, fuzzy=True)
    try:
        if objDate.tzinfo is None:
            objDate = LOCAL_TIMEZONE.localize(objDate)
    except AttributeError as e:
        raise ValueError(
            "Date %s which was converted to %s has no "
            "tzinfo attribute : %s" % (suspectedDate, objDate, e))

    objDate = utc.normalize(objDate)

    return objDate



def dict2List(inObj):
    '''given a dictionary, potentially with multiple sub dictionaries
       return a list of the dict keys and values
    '''
    if isinstance(inObj, dict):
        for key, value in inObj.iteritems():
            if isinstance(value, dict):
                for d in dict2List(value):
                    yield d
            elif isinstance(value, list):
                yield key.encode('ascii', 'ignore').lower()
                for l in dict2List(value):
                    yield l
            else:
                yield key.encode('ascii', 'ignore').lower()
                if isinstance(value, str):
                    yield value.lower()
                elif isinstance(value, unicode):
                    yield value.encode('ascii', 'ignore').lower()
                else:
                    yield value
    elif isinstance(inObj, list):
        for v in inObj:
            if isinstance(v, str):
                yield v.lower()
            elif isinstance(v, unicode):
                yield v.encode('ascii', 'ignore').lower()
            elif isinstance(v, list):
                for l in dict2List(v):
                    yield l
            elif isinstance(v, dict):
                for l in dict2List(v):
                    yield l
            else:
                yield v
    else:
        yield ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import os
from setuptools import setup, find_packages

readme_path = os.path.join(os.path.dirname(__file__), 'README.rst')
with open(readme_path) as readme_file:
    readme = readme_file.read()

history_path = os.path.join(os.path.dirname(__file__), 'HISTORY.rst')
with open(history_path) as history_file:
    history = history_file.read()

requirements = [
    'pip>=18.1',
    'bumpversion>=0.5.3',
    'wheel>=0.32.1',
    'watchdog>=0.9.0',
    'flake8>=3.5.0',
    'tox>=3.5.2',
    'coverage>=4.5.1',
    'Sphinx>=1.8.1',
    'twine>=1.12.1',
    'elasticsearch==6.3.1',
    'elasticsearch-dsl==6.3.1',
    'geoip2>=2.5.0',
    'pytz>=2017.3',
    'tzlocal>=1.4',
]

setup(
    author="Mozilla Infosec",
    author_email='infosec@mozilla.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Utilities shared throughout the MozDef codebase",
    install_requires=requirements,
    license='MPL-2.0',
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='mozdef_util',
    name='mozdef_util',
    packages=find_packages(),
    setup_requires=[],
    test_suite='tests',
    tests_require=[],
    url='https://github.com/mozilla/MozDef/tree/master/lib',
    version='2.0.1',
    zip_safe=False,
)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# Copyright (c) 2017 Mozilla Corporation


def AggregatedResults(input_results):
    converted_results = {
        'meta': {
            'timed_out': input_results.timed_out
        },
        'hits': [],
        'aggregations': {}
    }
    for hit in input_results.hits:
        hit_dict = {
            '_id': hit.meta.id,
            '_index': hit.meta.index,
            '_score': hit.meta.score,
            '_source': hit.to_dict()
        }
        converted_results['hits'].append(hit_dict)

    for agg_name, aggregation in input_results.aggregations.to_dict().iteritems():
        aggregation_dict = {
            'terms': []
        }
        for bucket in aggregation['buckets']:
            aggregation_dict['terms'].append({'count': bucket['doc_count'], 'key': bucket['key']})

        converted_results['aggregations'][agg_name] = aggregation_dict

    return converted_results



def isCEF(aDict):
    # determine if this is a CEF event
    # could be an event posted to the /cef http endpoint
    if 'endpoint' in aDict and aDict['endpoint'] == 'cef':
        return True
    # maybe it snuck in some other way
    # check some key CEF indicators (the header fields)
    if 'fields' in aDict and isinstance(aDict['fields'], dict):
        lowerKeys = [s.lower() for s in aDict['fields'].keys()]
        if 'devicevendor' in lowerKeys and 'deviceproduct' in lowerKeys and 'deviceversion' in lowerKeys:
            return True
    if 'details' in aDict and isinstance(aDict['details'], dict):
        lowerKeys = [s.lower() for s in aDict['details'].keys()]
        if 'devicevendor' in lowerKeys and 'deviceproduct' in lowerKeys and 'deviceversion' in lowerKeys:
            return True
    return False
